/*GPL*START*
 * 
 * Copyright (C) 1998 by Johannes Overmann <overmann@iname.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * *GPL*END*/  

#ifndef _mainwidget_h_
#define _mainwidget_h_

#include <qwidget.h>
#include <qpixmap.h>
#include <qlined.h>
#include <qchkbox.h>
#include <qtimer.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qradiobt.h>
#include <qpopmenu.h>

#include "machine.h"
#include "tassocarray.h"
#include "qjocakediagram.h"

class MainWidget: public QWidget {
   Q_OBJECT;   
 public:
   MainWidget(QWidget *parent=0, const char *name=0, WFlags f=0);
   ~MainWidget();

 public slots:
   void disp_groups();
   void disp_load();
   void disp_about();
   void disp_graph();
   void toggle_setup();
   void saveItems();
   void applyFromSetup();
   void resetSetup();
   void defaultSetup();
   void applyAndSaveItems();
   void updatedata();
   void updateview();
	void do_about();
	void want_quit();
   
   //sort slots
   void sort_load1();
   void sort_load2();
   void sort_load3();
   void sort_group();
   void sort_ram();
   void sort_name();
   void sort_arch();
   void resort();
   
   //popup slots
   void popact(int i);
   void pop1act(int i);
   void pop2act(int i);
   void pop3act(int i);
   void group_pop(const char *name, const QPoint& p);
   
 signals:
   void mach_changed();

 protected:
   bool event(QEvent *p);
   void mousePressEvent(QMouseEvent *p);
   void mouseReleaseEvent(QMouseEvent *p);
   void mouseMoveEvent(QMouseEvent *p);
   void paintEvent(QPaintEvent *p); 
      
 private:
   // methods:
   void createSetup();
   int findmach(const TString& name);
   int mustfindmach(const TString& name);
   void destroy_wid();
   void draw_load(bool only_graph=false);

   // setup:
   QWidget *setup;
   QCheckBox *b_autosave;
   QLineEdit *le_ruptime;
   QLineEdit *le_groups;
   QLineEdit *le_memory;
   QLineEdit *le_pop;
   QLineEdit *le_arc;
   QCheckBox *graph_grid;
   QCheckBox *graph_reverse;
   QLineEdit *graph_num;
   QLineEdit *le_update;
   QRadioButton *rb_graph_sep;
   QRadioButton *rb_graph_grp;
   QRadioButton *rb_graph_one;
   
   // main:
   enum MODE_T {M_ABOUT, M_LOAD, M_GRAPH, M_GROUPS} mode, dmode;
   enum SORT_T {S_NAME, S_GROUP, S_RAM, S_LOAD1, S_LOAD2, S_LOAD3, 
					 S_NAME_R, S_GROUP_R, S_RAM_R, S_LOAD1_R, S_LOAD2_R, S_LOAD3_R, 
					 S_RANDOM, S_ARCH, S_ARCH_R} sort;
   enum GRAPHMODE_T {GM_SEP, GM_GRP, GM_ONE};
   QTimer *timer;
   TArray<QWidget *> wid; // child widgets
   QBoxLayout *lay;      // layout
   QGridLayout *glay;      // layout
   bool no_resize;
   QPopupMenu *pop;
   QPopupMenu *pop1;
   QPopupMenu *pop2;
   QPopupMenu *pop3;
   TArray<QjoCakeDiagram *> group_cake_wid;
   TArray<TString> group_cake_grp;
   TArray<QRect> load_rect;
   TArray<TString> load_act;
	bool graph_drag;
	bool graph_value;
	QPixmap *about_pix;
	int about_phase;
	QWidget *about_lab;
	TArray<TString> load_pop_str;

   // data:
   TArray<Machine> m;
   TAssocArray<TString, int> grphist;  // number of group members with valid load
   TAssocArray<TString, double> grphistl;  // acc. load per group
   TArray<TString> grp;                // sorted list of group names
};

inline TString className(QRect) {return "QRect";}
inline void dump(QRect) {printf("a QRect");}
inline void dump(FILE *f, QRect) {fprintf(f, "a QRect");}

#endif

